from plone.z3cform.interfaces import IFormWrapper
from plone.z3cform.interfaces import IWrappedForm
from plone.z3cform.templates import FormTemplateFactory
from plone.z3cform.templates import ZopeTwoFormTemplateFactory
from plonetheme.nuplone.z3cform.interfaces import INuPloneFormLayer
from z3c.form.error import ErrorViewTemplateFactory
from z3c.form.interfaces import IForm

import os.path


TEMPLATE_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), "templates")

layout_factory = ZopeTwoFormTemplateFactory(
    os.path.join(TEMPLATE_DIR, "layout.pt"),
    form=IFormWrapper,
    request=INuPloneFormLayer,
)

wrapped_form_factory = FormTemplateFactory(
    os.path.join(TEMPLATE_DIR, "wrappedform.pt"),
    form=IWrappedForm,
    request=INuPloneFormLayer,
)

form_factory = ZopeTwoFormTemplateFactory(
    os.path.join(TEMPLATE_DIR, "form.pt"), form=IForm, request=INuPloneFormLayer
)

ErrorViewTemplate = ErrorViewTemplateFactory(os.path.join(TEMPLATE_DIR, "error.pt"))



from AccessControl.class_init import InitializeClass
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from Products.PluggableAuthService.interfaces.plugins import IChallengePlugin
from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from zope.interface import implementer


@implementer(IChallengePlugin)
class LoginChallenger(BasePlugin):
    """Simple login challenger which does nothing, allowing the Unauthorized
    exception view to be used instead."""

    meta_type = "NuPlone Login Challenger"
    id = "nuplone-challenger"

    def challenge(self, request, response):
        if not NuPloneSkin.providedBy(request):
            return False

        response.setStatus(403)
        response._locked_status = True
        return True


InitializeClass(LoginChallenger)



from plone.testing.zope import Browser
from plonetheme.nuplone.testing import NUPLONE_FUNCTIONAL_TESTING

import unittest


class PasswordResetTests(unittest.TestCase):
    layer = NUPLONE_FUNCTIONAL_TESTING

    def testNoKey(self):
        browser = Browser(self.layer["app"])
        browser.open("%s/reset-password" % self.layer["portal"].absolute_url())
        browser.getForm("form").submit()

    def testInvalidKey(self):
        from zExceptions import NotFound

        # The browser tests was replaced because of this issue:
        # - https://github.com/plone/plone.testing/issues/83
        #
        # browser = Browser(self.layer["app"])
        # browser.handleErrors = False
        # url = "%s/reset-password/bogus" % self.layer["portal"].absolute_url()
        with self.assertRaises(NotFound):
            self.layer["portal"].restrictedTraverse("reset-password/bogus")
            # browser.open(url)

    def testDoubleKeys(self):
        from zExceptions import NotFound

        # The browser tests was replaced because of this issue:
        # - https://github.com/plone/plone.testing/issues/83
        #
        # browser = Browser(self.layer["app"])
        # browser.handleErrors = False
        # url = "%s/reset-password/one/two" % self.layer["portal"].absolute_url()
        with self.assertRaises(NotFound):
            self.layer["portal"].restrictedTraverse("reset-password/one/two")
            # browser.open(url)



from Acquisition import aq_base
from Acquisition import aq_chain
from Acquisition import aq_inner
from Acquisition import aq_parent
from Products.Five import BrowserView
from zope.browser.interfaces import IBrowserView

import logging
import zExceptions


log = logging.getLogger(__name__)


class Error(BrowserView):
    def update(self):
        self.exception = aq_inner(self.context)
        self.context = context = aq_parent(self)
        if IBrowserView.providedBy(context):
            # NotFound errors can have extra aq wrapping
            self.context = context = aq_parent(context)
        log.debug("%r -> %r", self.exception, context)

    def __call__(self):
        self.update()
        return super().__call__()


class Unauthorized(Error):
    def authenticate(self):
        """Try to authenticate the user manually, since ZPublisher dropped the
        user when it failed to validate access."""
        for parent in aq_chain(aq_inner(self.context)):
            if hasattr(aq_base(parent), "acl_users"):
                uf = parent.acl_users
                try:
                    return uf.validate(self.request, None, roles=["Anonymous"])
                except zExceptions.Unauthorized:
                    pass

    def update(self):
        super().update()
        self.authenticate()



from plone.autoform.form import AutoExtensibleForm
from plone.supermodel import model
from plonetheme.nuplone import MessageFactory as _
from z3c.form import form
from zope import schema


class ExternalLinkSchema(model.Schema):
    URL = schema.URI(title=_("label_url", default="URL"), required=True)

    title = schema.TextLine(title=_("label_title", default="Title"), required=False)

    new_window = schema.Bool(
        title=_("label_new_window", default="Open link in new window"), default=True
    )


class EditLink(AutoExtensibleForm, form.EditForm):
    ignoreContext = True
    schema = ExternalLinkSchema



from AccessControl import getSecurityManager
from Acquisition import aq_inner
from plonetheme.nuplone import MessageFactory as _
from plonetheme.nuplone.utils import isAnonymous
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage

import logging


log = logging.getLogger(__name__)


class Login(BrowserView):
    def homeUrl(self, user):
        """Return a suitable `home' for a user if came_from is unset or
        invalid."""
        put = getToolByName(self.context, "portal_url")
        return put.getPortalObject().absolute_url()

    def update(self):
        user = getSecurityManager().getUser()
        self.came_from = self.request.get("came_from")
        if not self.came_from:
            self.came_from = self.request.environ.get("HTTP_REFERER")
        elif "@@reset-password" in self.came_from:
            self.came_from = None

        if self.came_from:
            if ":" not in self.came_from:
                # Mostly for mechanize/testbrowser which starts with a bogus
                # 'localhost' as referer.
                self.came_from = None
            else:
                put = getToolByName(self.context, "portal_url")
                if not put.isURLInPortal(self.came_from):
                    self.came_from = None

        self.anonymous = isAnonymous(user)
        self.action = "%s/@@login" % aq_inner(self.context).absolute_url()

        if not self.anonymous and "login_attempt" in self.request:
            flash = IStatusMessage(self.request).addStatusMessage
            flash(_("message_logged_in", default="You have been logged in."), "success")
            mt = getToolByName(self.context, "portal_membership")
            mt.loginUser(self.request)
            next = self.came_from or self.homeUrl(user)
            self.request.response.redirect(next)

        self.failed = self.anonymous and "login_attempt" in self.request

    def __call__(self):
        self.update()
        return super().__call__()


class Logout(BrowserView):
    def __call__(self):
        flash = IStatusMessage(self.request).addStatusMessage
        if not isAnonymous():
            mt = getToolByName(self.context, "portal_membership")
            mt.logoutUser(self.request)
            flash(
                _("message_logged_out", default="You have been logged out."),
                "success",
            )
        else:
            flash(
                _(
                    "message_already_logged_out",
                    default="You were already logged out.",
                ),
                "notice",
            )
        return self.request.response.redirect(aq_inner(self.context).absolute_url())



from chameleon.utils import Markup
from plonetheme.nuplone.tiles.tile import getTile
from Products.PageTemplates.Expressions import getTrustedEngine
from zope.tales.expressions import StringExpr

import logging


log = logging.getLogger(__name__)


class TileExpression(StringExpr):
    def __call__(self, econtext):
        name = super().__call__(econtext)
        context = econtext.vars["context"]
        request = econtext.vars["request"]

        tile = getTile(context, request, name)
        if tile is None:  # XXX Use custom exception?
            log.warning("Request for unknown tile %s", name)
            return ""
        return Markup(tile())


engine = getTrustedEngine()
engine.registerType("tile", TileExpression)



from zope.dottedname.resolve import resolve


# I am not going to document this. This is a essentially workaround
# http://code.google.com/p/dexterity/issues/detail?id=123


def FieldWidgetFactory(factory, **kw):
    if isinstance(factory, str):
        factory = resolve(factory)

    def wrapper(field, request):
        widget = factory(field, request)
        for key, value in kw.items():
            setattr(widget, key, value)
        return widget

    return wrapper



from Acquisition import aq_inner
from plonetheme.nuplone.tiles.tile import AppConfigTile
from plonetheme.nuplone.tiles.tile import getTile
from plonetheme.nuplone.utils import viewType
from zExceptions import NotFound

import re


_compiles = {}


class GroupTile(AppConfigTile):
    TILE_EXPR = re.compile(r"(?P<name>\S+)\s*(?:\[(?P<condition>.*)\])?")
    tiles = []

    def update(self):
        self.tiles = []
        self.tag = self.data.get("wrapper", "tal:block")
        tiles = self.data.get("tiles", "").strip().splitlines()
        context = aq_inner(self.context)
        request = self.request
        globals = None
        for tile in tiles:
            result = self.TILE_EXPR.match(tile)
            if not result:
                continue
            condition = result.group("condition")
            if condition:
                compiled = _compiles.get(condition)
                if compiled is None:
                    compiled = _compiles[condition] = compile(
                        condition, "<string>", "eval"
                    )
                if globals is None:
                    globals = dict(
                        context=context,
                        request=request,
                        view_type=viewType(context, request),
                    )
                if not eval(compiled, globals):  # nosec  # TODO: get rid of eval
                    continue
            tile = getTile(context, request, result.group("name"))
            self.tiles.append(tile)

    def __call__(self):
        if isinstance(self.context, NotFound):
            return ""
        self.update()
        if not self.tiles:
            return ""
        result = self.index()
        if not result.strip():
            return ""
        config = self.data
        wrapper = config.get("wrapper")
        if wrapper:
            prefix = ["<%s" % wrapper]
            if config.get("id"):
                prefix.append(' id="%s"' % config["id"])
            if config.get("class"):
                prefix.append(' class="%s"' % config["class"])
            prefix.append(">")
            prefix = "".join(prefix)
            postfix = "</%s>" % wrapper
            result = f"{prefix}{result}{postfix}"
        return result



from Acquisition import aq_inner
from plone import api
from plone.autoform import directives
from plone.autoform.form import AutoExtensibleForm
from plone.supermodel import model
from plonetheme.nuplone import MessageFactory as _
from plonetheme.nuplone.utils import createEmailTo
from plonetheme.nuplone.z3cform.form import FieldWidgetFactory
from Products.CMFCore.utils import getToolByName
from Products.MailHost.MailHost import MailHostError
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import form
from z3c.form.button import buttonAndHandler
from zope import schema
from zope.i18n import translate

import logging


log = logging.getLogger(__name__)

TextSpan7 = FieldWidgetFactory("z3c.form.browser.text.TextFieldWidget", klass="span-7")
TextLines4Rows = FieldWidgetFactory(
    "z3c.form.browser.textlines.TextLinesFieldWidget", rows=4
)


class IContact(model.Schema):
    name = schema.TextLine(
        title=_("label_your_name", default="Your name"), required=True
    )
    directives.widget(name="plonetheme.nuplone.browser.contact.TextSpan7")

    email = schema.ASCIILine(
        title=_("label_email", default="Email address"), required=True
    )
    directives.widget(email="plonetheme.nuplone.browser.contact.TextSpan7")

    subject = schema.TextLine(
        title=_("label_subject", default="Subject"), required=True
    )
    directives.widget(subject="plonetheme.nuplone.browser.contact.TextSpan7")

    message = schema.Text(
        title=_("label_contact_text", default="Your message"), required=True
    )
    directives.widget(message="plonetheme.nuplone.browser.contact.TextLines4Rows")


class ContactForm(AutoExtensibleForm, form.Form):
    ignoreContext = True
    schema = IContact
    label = _("header_contact", default="Contact")
    default_fieldset_label = None

    @property
    def email_from_name(self):
        return api.portal.get_registry_record("plone.email_from_name")

    @property
    def email_from_address(self):
        return api.portal.get_registry_record("plone.email_from_address")

    @buttonAndHandler(_("button_send", default="Send"), name="send")
    def handleSend(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        subject = _(
            "contact_mail_subject",
            default="Contact request: ${subject}",
            mapping=data,
        )
        subject = translate(subject, context=self.request)

        email = createEmailTo(
            data["name"],
            data["email"],
            self.email_from_name,
            self.email_from_address,
            subject,
            data["message"],
        )
        mh = getToolByName(self.context, "MailHost")
        flash = IStatusMessage(self.request).addStatusMessage
        try:
            mh.send(email)
        except MailHostError as e:
            log.error(
                "MailHost error sending contact form for %s: %s", data["email"], e
            )
            flash(
                _(
                    "error_contactmail",
                    "An error occured while processing your contact request. Please try again later.",  # noqa: E501
                ),
                "error",
            )
            return
        except OSError as e:
            log.error(
                "Socket error sending contact form for %s: %s", data["email"], e[1]
            )
            flash(
                _(
                    "error_contactmail",
                    "An error occured while processing your contact request. Please try again later.",  # noqa: E501
                ),
                "error",
            )
            return

        flash(_("Message sent"), "success")
        self.request.response.redirect(aq_inner(self.context).absolute_url())



from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from plonetheme.nuplone.testing import NUPLONE_INTEGRATION_TESTING
from zope.interface import alsoProvides

import unittest
import zope.component


class SiteMenuIntegrationTests(unittest.TestCase):
    layer = NUPLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.portal.REQUEST
        alsoProvides(self.request, NuPloneSkin)

    def test_settings_url(self):
        """Check that the sitemenu view provides a settings_url attribute."""
        view = zope.component.getMultiAdapter(
            (self.portal, self.request),
            name="sitemenu",
        )
        self.assertEqual(view.settings_url, "http://nohost/plone/@@settings")


class SiteMenuTests(unittest.TestCase):
    layer = NUPLONE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.portal.REQUEST
        alsoProvides(self.request, NuPloneSkin)
        login(self.layer["app"], SITE_OWNER_NAME)

        # Create two folders
        self.portal.invokeFactory("Folder", "source_folder")
        self.source_folder = self.portal["source_folder"]

        self.portal.invokeFactory("Folder", "dest_folder")
        self.dest_folder = self.portal["dest_folder"]

        # Create an object to be copied
        self.source_folder.invokeFactory("Document", "doc")

    def testActions(self):
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ("Contributor",))
        # Copy the object
        cp = self.source_folder.manage_copyObjects(ids=["doc"])
        self.request["__cp"] = cp

        # We cannot paste the object in the site root
        v = zope.component.getMultiAdapter((self.portal, self.request), name="sitemenu")
        self.assertIsNone(v.organise())

    def test_adding_to_same_category(self):
        from plonetheme.nuplone.skin.sitemenu import Sitemenu

        sitemenu = Sitemenu(None, None)
        sitemenu.factories = lambda: {
            "title": "cat1",
            "children": [{"title": "sub1.1"}, {"title": "sub1.2"}],
        }
        sitemenu.organise = lambda: {"title": "cat1", "children": [{"title": "sub1.3"}]}

        menu = sitemenu.actions
        self.assertEqual(len(menu["children"]), 1)
        self.assertEqual(menu["children"][0]["title"], "cat1")
        self.assertEqual(len(menu["children"][0]["children"]), 3)
        self.assertEqual(menu["children"][0]["children"][0]["title"], "sub1.1")
        self.assertEqual(menu["children"][0]["children"][1]["title"], "sub1.2")
        self.assertEqual(menu["children"][0]["children"][2]["title"], "sub1.3")

    def test_add_submenu(self):
        from plonetheme.nuplone.skin.sitemenu import Sitemenu

        sitemenu = Sitemenu(None, None)
        menu = [
            {"title": "cat1", "children": [{"title": "sub1.1"}]},
            {"title": "cat2", "children": [{"title": "sub2.1"}]},
        ]
        sitemenu.add_submenu(menu, {"title": "cat1", "children": [{"title": "sub1.2"}]})
        self.assertEqual(len(menu), 2)
        self.assertEqual(menu[0]["title"], "cat1")
        self.assertEqual(len(menu[0]["children"]), 2)
        self.assertEqual(menu[0]["children"][0]["title"], "sub1.1")
        self.assertEqual(menu[0]["children"][1]["title"], "sub1.2")
        self.assertEqual(menu[1]["title"], "cat2")
        self.assertEqual(len(menu[1]["children"]), 1)
        self.assertEqual(menu[1]["children"][0]["title"], "sub2.1")



from plone import api
from plone.tiles.interfaces import ITile
from plone.tiles.interfaces import ITileDataManager
from plone.tiles.tile import Tile
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.interface import Interface

import json
import logging


log = logging.getLogger(__name__)


class IAppConfigTile(ITile):
    """A tile with its configured stored in the application config."""


@implementer(IAppConfigTile)
class AppConfigTile(Tile):
    pass


@implementer(ITileDataManager)
@adapter(IAppConfigTile)
class AppConfigTileDataManager:
    def __init__(self, tile):
        self.tile = tile

    def get(self):
        try:
            return json.loads(
                api.portal.get_registry_record(
                    f"plonetheme.nuplone.appconfigtile_{self.tile.id}",
                    default="{}",
                )
            )
        except Exception:
            log.exception("Cannot get configuration for %r", self.tile.id)
            return {}

    def set(self, data):
        raise NotImplementedError


def getTile(context, request, name):
    config = json.loads(
        api.portal.get_registry_record(
            f"plonetheme.nuplone.appconfigtile_{name}", default="{}"
        )
        or "{}"
    )
    type = config.get("type", name)
    tile = queryMultiAdapter((context, request), Interface, name=type)
    if tile is None:
        log.warning(
            "Detected reference to non-existing tile '%s' for context %r", name, context
        )
        return None

    if IAppConfigTile.providedBy(tile):
        tile.id = name

    return tile



from Acquisition import aq_inner
from OFS.interfaces import ICopyContainer
from plone import api
from plone.memoize.view import memoize_contextless
from plone.protect.utils import addTokenToUrl
from plonetheme.nuplone import MessageFactory as _
from plonetheme.nuplone.utils import getFactoriesInContext
from Products.CMFCore.ActionInformation import ActionInfo
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView


class Sitemenu(BrowserView):
    @property
    @memoize_contextless
    def tools_view(self):
        return api.content.get_view(
            "tools",
            self.context,
            self.request,
        )

    @property
    def settings_url(self):
        return "%s/@@settings" % self.tools_view.navroot_url

    @property
    def view_type(self):
        return self.request.get("view_type", "view")

    def add_submenu(self, menu, submenu):
        """Add a submenu to the menu by extending or adding a category."""
        for item in menu:
            if item["title"] == submenu["title"]:
                item["children"].extend(submenu["children"])
                return
        menu.append(submenu)

    @property
    def actions(self):
        """Helper method to generate the contents for the actions menu."""
        menu = {"title": _("menu_actions", default="Actions")}
        children = menu["children"] = []
        submenu = self.factories()
        if submenu:
            self.add_submenu(children, submenu)
        submenu = self.organise()
        if submenu:
            self.add_submenu(children, submenu)

        if children:
            return menu
        else:
            return None

    def factories(self):
        """Helper method to generate the menu items for creating new content.

        If no content can be created None is returned.
        """
        menu = {"title": _("menu_add_new", default="Add new")}
        children = menu["children"] = []
        actions = getFactoriesInContext(self.context)
        actions.sort(key=lambda x: x.title)
        for action in actions:
            children.append(
                {
                    "title": action.title,
                    "description": action.description,
                    "url": action.url,
                }
            )
        if children:
            return menu
        else:
            return None

    def organise(self):
        """Helper method to generate the menu items for organising content
        (copy/paste/etc.).

        If no organising actions are available None is returned.
        """
        context = aq_inner(self.context)
        context_url = context.absolute_url()
        is_root = ISiteRoot.providedBy(context)
        pa = getToolByName(context, "portal_actions")
        actions = pa.listActions(
            object=context, categories=("object_buttons",), ignore_categories=None
        )
        ec = pa._getExprContext(context)
        actions = [ActionInfo(action, ec) for action in actions]

        menu = {"title": _("menu_organise", default="Organise")}
        children = menu["children"] = []
        for a in actions:
            if a["visible"] and a["allowed"] and a["available"] and not is_root:
                if a["id"] == "copy" and context.cb_isCopyable():
                    children.append(
                        {
                            "title": _("menu_copy", default="Copy"),
                            "url": "%s/@@copy" % context_url,
                        }
                    )

                elif a["id"] == "cut" and context.cb_isMoveable():
                    children.append(
                        {
                            "title": _("menu_cut", default="Cut"),
                            "url": "%s/@@cut" % context_url,
                        }
                    )

                elif a["id"] == "paste" and ICopyContainer.providedBy(context):
                    url = addTokenToUrl("%s/@@paste" % context_url)
                    children.append(
                        {
                            "title": _("menu_paste", default="Paste"),
                            "url": url,
                        }
                    )
                elif a["id"] == "delete":
                    children.append(
                        {
                            "title": _("menu_delete", default="Delete"),
                            "url": "%s/@@delete" % context_url,
                        }
                    )
        if children:
            return menu



from Acquisition import aq_inner
from Acquisition import aq_parent
from OFS.CopySupport import CopyError
from plonetheme.nuplone import MessageFactory as _
from plonetheme.nuplone.utils import checkPermission
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from ZODB.POSException import ConflictError
from zope.component import getMultiAdapter

import zExceptions


class Copy(BrowserView):
    def __call__(self):
        context = aq_inner(self.context)
        container = aq_parent(context)
        if not context.cb_isCopyable():
            raise zExceptions.Unauthorized

        container.manage_copyObjects(context.getId(), self.request)
        flash = IStatusMessage(self.request).addStatusMessage
        flash(_("message_copy_sucess", default="Copied"), "notice")
        self.request.response.redirect(context.absolute_url())


class Cut(BrowserView):
    def __call__(self):
        context = aq_inner(self.context)
        container = aq_parent(context)
        flash = IStatusMessage(self.request).addStatusMessage

        try:
            container.manage_cutObjects(context.getId(), self.request)
            flash(_("message_cut_success", default="Cut."), "notice")
        except CopyError:
            flash(
                _(
                    "message_cut_invalid",
                    default="It is not possible to move this object.",
                ),
                "error",
            )

        self.request.response.redirect(context.absolute_url())


class Paste(BrowserView):
    def __call__(self):
        context = aq_inner(self.context)
        flash = IStatusMessage(self.request).addStatusMessage
        if not context.cb_dataValid():
            raise zExceptions.Unauthorized

        try:
            context.manage_pasteObjects(REQUEST=self.request)
            flash(_("message_paste_succes", default="Pasted"), "success")
        except ConflictError:
            raise
        except ValueError:
            flash(
                _(
                    "message_paste_valueerror",
                    default="You can not paste the copied data here.",
                ),
                "error",
            )
        except zExceptions.Unauthorized:
            flash(
                _(
                    "message_paste_unauthorized",
                    default="You are not allowed to paste here.",
                ),
                "error",
            )
        except CopyError as e:
            if "Insufficient Privileges" in e.message:
                raise zExceptions.Unauthorized
            flash(
                _(
                    "message_paste_generic",
                    default="No valid data found in the clipboard.",
                ),
                "error",
            )

        self.request.response.redirect(context.absolute_url())


class Delete(BrowserView):
    def verify(self, container, context):
        if not checkPermission(container, "Delete objects"):
            raise zExceptions.Unauthorized

        return True

    def post(self):
        action = self.request.form.get("action", "cancel")
        context = aq_inner(self.context)
        flash = IStatusMessage(self.request).addStatusMessage

        if action == "cancel":
            flash(_("message_delete_cancel", default="Deletion cancelled"), "notice")
            self.request.response.redirect(context.absolute_url())
        elif action == "delete":
            authenticator = getMultiAdapter(
                (self.context, self.request), name="authenticator"
            )
            if not authenticator.verify():
                raise zExceptions.Unauthorized

            container = aq_parent(context)
            container.manage_delObjects([context.getId()])
            flash(_("message_delete_success", default="Object removed"), "success")
            self.request.response.redirect(container.absolute_url())

    def __call__(self):
        context = aq_inner(self.context)
        container = aq_parent(context)

        if not self.verify(container, context):
            return

        if self.request.method == "POST":
            return self.post()
        return super().__call__()



from Acquisition import aq_inner
from plonetheme.nuplone import MessageFactory as _
from plonetheme.nuplone.utils import setLanguage
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage


class SwitchLanguage(BrowserView):
    def __call__(self):
        flash = IStatusMessage(self.request).addStatusMessage
        if setLanguage(self.request, self.context):
            flash(_("message_switch_language", default="Language updated"), "success")
        else:
            flash(
                _(
                    "message_switch_language_error",
                    default="Failed to switch language",
                ),
                "error",
            )

        next_url = self.request.get("came_from")
        if not next_url:
            next_url = self.request.environ.get("HTTP_REFERER")
        put = getToolByName(self.context, "portal_url")
        if not next_url or not put.isURLInPortal(next_url):
            next_url = aq_inner(self.context).absolute_url()

        self.request.response.redirect(next_url)



from Acquisition import aq_inner
from plone import api
from plone.tiles import Tile
from plonetheme.nuplone.utils import getNavigationRoot
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import typesToList
from zExceptions import NotFound


class TabsTile(Tile):
    def getRoot(self):
        return getNavigationRoot(self.context)

    def update(self):
        context = aq_inner(self.context)
        contextUrl = context.absolute_url()
        navroot = self.getRoot()
        navrootPath = "/".join(navroot.getPhysicalPath())
        use_view_types = api.portal.get_registry_record(
            "plone.types_use_view_action_in_listings", default=[]
        )
        query = {}
        query["path"] = dict(query=navrootPath, depth=1)
        query["portal_type"] = typesToList(context)
        query["sort_on"] = "getObjPositionInParent"
        query["sort_order"] = "asc"
        query["is_default_page"] = False

        catalog = getToolByName(context, "portal_catalog")
        results = [
            {
                "id": brain.id,
                "title": brain.Title,
                "url": "%s/view" % brain.getURL()
                if brain.portal_type in use_view_types
                else brain.getURL(),
                "class": None,
            }
            for brain in catalog.searchResults(query)
            if not brain.exclude_from_nav
        ]
        current = sorted(
            [
                (len(result["url"]), result["id"])
                for result in results
                if contextUrl.startswith(result["url"])
            ]
        )
        if current:
            current = current[0][1]
            for result in results:
                if result["id"] == current:
                    result["class"] = "current"
                    break

        self.tabs = results
        self.home_url = navroot.absolute_url()

    def __call__(self):
        if isinstance(self.context, NotFound):
            return ""
        self.update()
        return self.index()



from AccessControl import getSecurityManager
from Acquisition import aq_chain
from Acquisition import aq_inner
from email.header import Header
from email.mime.text import MIMEText
from plone.app.layout.navigation.interfaces import INavigationRoot
from plonetheme.nuplone import MessageFactory as _
from Products.CMFCore.ActionInformation import ActionInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from z3c.form.interfaces import IAddForm
from z3c.form.interfaces import IEditForm

import collections
import email.utils as emailutils
import logging


log = logging.getLogger(__name__)


def getPortal(context):
    for obj in aq_chain(aq_inner(context)):
        if IPloneSiteRoot.providedBy(obj):
            return obj


def getNavigationRoot(context):
    for obj in aq_chain(aq_inner(context)):
        if INavigationRoot.providedBy(obj):
            break
    return obj


def checkPermission(context, permission):
    return getSecurityManager().checkPermission(permission, context)


def isAnonymous(user=None):
    if user is None:
        user = getSecurityManager().getUser()
    return user is None or user.getUserName() == "Anonymous User"


def viewType(context, request):
    """Determine what type of view the user is looking at.

    This returns one of three options: ``view`` for normal object views,
    ``add`` for add forms, ``edit`` for edit forms, and ``other`` for
    all other types.
    """

    view = request.other.get("PUBLISHED")
    if view is not None:
        if IAddForm.providedBy(view):
            return "add"
        if IEditForm.providedBy(view):
            return "edit"

    for url in ["ACTUAL_URL", "VIRTUAL_URL", "URL"]:
        current_url = request.get(url)
        if current_url is not None:
            break
    else:
        return "view"

    if current_url.endswith("/"):
        current_url = current_url[:-1]

    if current_url.endswith("@@edit"):
        return "edit"
    elif "/++add++" in current_url:
        return "add"

    try:
        if current_url == aq_inner(context).absolute_url():
            return "view"
    except AttributeError:
        # object has no absolute_url (None or Not Found)
        pass

    return "other"


class SimpleLiteral(str):
    def __html__(self):
        return str(self)


def setLanguage(request, context, lang=None):
    """Switch Plone to another language. If no language is given via the `lang`
    parameter the language is taken from a `language` request parameter. If a
    dialect was chosen but is not available the main language is used instead.
    If the main language is also not available nothing is done and ``False`` is
    returned.

    It is safe to call this method before rendering a template: it will
    immediately switch the current language. That means there is no need
    to trigger a redirect to force a new HTTP request.
    """
    if lang is None:
        lang = request.form.get("language")
    if not lang:
        return

    lang = lang.lower()
    lt = getToolByName(context, "portal_languages")
    res = lt.setLanguageCookie(lang=lang, request=request)
    if res is None and "-" in lang:
        lang = lang.split("-")[0]
        res = lt.setLanguageCookie(lang=lang, request=request)
        if res is None:
            log.warning("Failed to switch language to %s", lang)
            return False

    # In addition to setting the cookie also update the PTS language.
    # This effectively switches Plone over to the new language without
    # requiring a new HTTP request.
    request["LANGUAGE"] = lang
    binding = request.get("LANGUAGE_TOOL", None)
    if binding is not None:
        binding.LANGUAGE = lang

    return True


FactoryInfo = collections.namedtuple("FactoryInfo", "id title description url")


def getFactoriesInContext(context):
    """Return a list of all factories available to the current user at the
    given location."""
    context = aq_inner(context)
    ftis = context.allowedContentTypes()
    if not ftis:
        return []

    tt = getToolByName(context, "portal_types")
    ec = tt._getExprContext(context)
    actions = [ActionInfo(fti, ec) for fti in ftis]
    actions = [
        FactoryInfo(
            action.get("id"),
            action.get("title") or action.get("id"),
            action.get("description") or None,
            action["url"],
        )
        for action in actions
    ]
    return actions


def createEmailTo(
    sender_name,
    sender_email,
    recipient_name,
    recipient_email,
    subject,
    body,
    format="plain",
):
    """Create an :obj:`email.MIMEText.MIMEtext` instance for an email.

    This method will take care of adding addings a date header and
    message ID to the email, as well as quoting of non-ASCII content.
    """
    if isinstance(body, str):
        mail = MIMEText(body.encode("utf-8"), format, "utf-8")
    else:
        mail = MIMEText(body, format)

    if sender_name:
        mail["From"] = emailutils.formataddr((sender_name, sender_email))
    else:
        mail["From"] = sender_email
    if recipient_name:
        mail["To"] = emailutils.formataddr((recipient_name, recipient_email))
    else:
        mail["To"] = recipient_email
    mail["Subject"] = Header(subject.encode("utf-8"), "utf-8")
    mail["Date"] = emailutils.formatdate(localtime=True)
    mail.set_param("charset", "utf-8")

    return mail


def formatDate(request, date, length="long"):
    """Wrapper aound the zope.i18n date formatter which does not abort on
    pre-1900 dates."""
    if date.year < 1900:
        return _("date_to_early", default="<pre-1900-date>")
    return request.locale.dates.getFormatter("date", length).format(date)


def formatTime(request, time, length=None):
    return request.locale.dates.getFormatter("time", length).format(time)


def formatDatetime(request, timestamp, length="long"):
    """Wrapper aound the zope.i18n datetime formatter which does not abort on
    pre-1900 dates."""
    if timestamp.year < 1900:
        return _("date_to_early", default="<pre-1900-date>")
    if length == "long":
        return _(
            "format_datetime",
            default="${date} at ${time}",
            mapping=dict(
                date=formatDate(request, timestamp, "long"),
                time=formatTime(request, timestamp, "short"),
            ),
        )
    return request.locale.dates.getFormatter("dateTime", length).format(timestamp)
